import 'dart:ui';

import 'package:dotup_flutter_mockups/src/mockup_descriptor.dart';
import 'package:flutter/material.dart';

import 'mockup_page_descriptor.dart';
import 'mockup_presenter.dart';
import 'mockup_splash_screen.dart';
import 'settings/settings_controller.dart';
import 'settings/settings_view.dart';

class DesktopScrollBehavior extends MaterialScrollBehavior {
  @override
  Set<PointerDeviceKind> get dragDevices => {
        PointerDeviceKind.touch,
        PointerDeviceKind.mouse,
      };
}

class DotupFlutterMockupsApp extends StatelessWidget {
  const DotupFlutterMockupsApp({
    Key? key,
    required this.settingsController,
    required this.mockupDescriptor,
  }) : super(key: key);

  final SettingsController settingsController;

  final MockupDescriptor mockupDescriptor;

  static final messenger = GlobalKey<ScaffoldMessengerState>();

  @override
  Widget build(BuildContext context) {
    // final controller = MockupController(pages: mocks);

    return AnimatedBuilder(
      animation: settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
          scaffoldMessengerKey: messenger,
          scrollBehavior: DesktopScrollBehavior(),
          debugShowCheckedModeBanner: false,
          // Providing a restorationScopeId allows the Navigator built by the
          // MaterialApp to restore the navigation stack when a user leaves and
          // returns to the app after it has been killed while running in the
          // background.
          restorationScopeId: 'app',

          theme: ThemeData(),
          darkTheme: ThemeData.dark().copyWith(primaryColor: TOB_COLOR, primaryColorDark: TOB_COLOR),
          themeMode: SettingsController.themeMode,

          onGenerateRoute: (RouteSettings routeSettings) {
            // final ft = FadeTransition(opacity: 1);

            switch (routeSettings.name) {
              case '/':
                return MaterialPageRoute<void>(
                  settings: routeSettings,
                  builder: (BuildContext context) {
                    return MockupSplashScreen(
                      onTap: (showPageInfos) => Navigator.of(context).restorablePushReplacementNamed('mockup', arguments: showPageInfos),
                    );
                  },
                );

              case SettingsView.routeName:
                return MaterialPageRoute<void>(
                  settings: routeSettings,
                  builder: (BuildContext context) {
                    return SettingsView(controller: settingsController);
                  },
                );

              default:
                return PageRouteBuilder(
                  pageBuilder: (context, animation, secondaryAnimation) {
                    final showPageInfos = routeSettings.arguments as bool;
                    return MockupPresenter(
                      pages: mockupDescriptor.pages,
                      showPageInfos: showPageInfos,
                      // controller: MockupController(pages: mocks),
                    );
                  },
                  transitionsBuilder: (context, animation, secondaryAnimation, child) => FadeTransition(
                    opacity: animation,
                    child: child,
                  ),
                  transitionDuration: Duration(milliseconds: 1000),
                );

              // return  FadeInPageRoute<void>(
              //   settings: routeSettings,
              //   builder: (BuildContext context) {
              //     return OperationListView(controller: c);
              //   },
              // );
            }
          },
        );
      },
    );
  }
}

class FadeInPageRoute<T> extends MaterialPageRoute<T> {
  FadeInPageRoute({
    RouteSettings? settings,
    required WidgetBuilder builder,
  }) : super(settings: settings, builder: builder);

  @override
  // TODO: implement barrierColor
  Color get barrierColor => Colors.black;

  @override
  String get barrierLabel => '';

  @override
  Widget buildPage(BuildContext context, Animation<double> animation, Animation<double> secondaryAnimation) {
    return FadeTransition(
      opacity: animation,
      child: builder(context),
    );
  }

  @override
  bool get maintainState => true;

  @override
  Duration get transitionDuration => Duration(milliseconds: 1500);
}
