import 'package:dotup_device_simulator/dotup_device_simulator.dart';
import 'package:dotup_flutter_mockups/src/dotup_flutter_mockups_app.dart';
import 'package:dotup_flutter_mockups/src/mockup_controller.dart';
import 'package:dotup_flutter_mockups/src/mockup_popup_menu.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'mockup_page_descriptor.dart';
import 'mockup_page_list.dart';

class MockupPresenter extends StatefulWidget {
  const MockupPresenter({
    Key? key,
    required this.pages,
    required this.showPageInfos,
  }) : super(key: key);

  final List<MockupPageDescriptor> pages;
  final bool showPageInfos;

  @override
  _MockupPresenterState createState() => _MockupPresenterState();

  static void showPageInfo(MockupController controller) {
    DotupFlutterMockupsApp.messenger.currentState!.hideCurrentSnackBar();
    // ScaffoldMessenger.of(context).hideCurrentSnackBar();
//                    ScaffoldMessenger.of(context).showSnackBar(
    DotupFlutterMockupsApp.messenger.currentState!.showSnackBar(
      SnackBar(
        // margin: EdgeInsets.all(0),
        // padding: EdgeInsets.all(0),
        elevation: 4,
        backgroundColor: Colors.grey.shade900,
        behavior: SnackBarBehavior.floating,
        content: Material(
          color: Colors.grey.shade900,
          child: Padding(
            padding: const EdgeInsets.all(0.0),
            child: Card(
              child: ListTile(
                title: Text(controller.currentPage.title),
                contentPadding: EdgeInsets.all(16),
                leading: CircleAvatar(
                  child: Text(controller.currentPage.number.toString()),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class _MockupPresenterState extends State<MockupPresenter> {
  late final MockupController controller; // = MockupController(pages: []);
  // late bool menuVisible;
  // late bool showPageInfos;

  @override
  void initState() {
    super.initState();
    // menuVisible = false;
    // showPageInfos = true;
    controller = MockupController(pages: widget.pages, pageController: PageController());
    // controller.toPage(1);
  }

  @override
  Widget build(BuildContext context) {
    // void _toNextPage() => pageController.nextPage(duration: const Duration(milliseconds: 300), curve: Curves.easeIn);
    // void _toPage(int index) => pageController.animateToPage(index, duration: const Duration(milliseconds: 300), curve: Curves.easeIn);

    return ChangeNotifierProvider<MockupController>(
      create: (context) => controller,
      child: Consumer<MockupController>(
        builder: (context, value, child) {
          return GestureDetector(
            onLongPress: () {
              controller.showPageMenu();
            },
            child: PageView.builder(
              pageSnapping: true,
              itemCount: controller.pages.length,
              controller: controller.pageController,
              itemBuilder: (context, index) {
                if (widget.showPageInfos && controller.currentPageIndex != index && controller.pageMenuVisible == false) {
                  // print('addpostback');
                  WidgetsBinding.instance!.addPostFrameCallback((_) {
                    MockupPresenter.showPageInfo(controller);
                  });
                }

                controller.setCurrentIndex(index);
                // if (controller.pageInfosVisible) {
                //   Future.delayed(Duration(seconds: 3), () {
                //     if (mounted) controller.hidePageInfos();
                //   });
                // }

                return Stack(
                  children: [
                    MediaQuery(data: DeviceSimulator.mediaQueryData, child: controller.getBuilder(index)(context)),

                    // AnimatedOpacity(
                    //   // If the widget is visible, animate to 0.0 (invisible).
                    //   // If the widget is hidden, animate to 1.0 (fully visible).
                    //   opacity: menuVisible ? 1.0 : 0.0,
                    //   duration: const Duration(milliseconds: 500),
                    //   // The green box must be a child of the AnimatedOpacity widget.
                    //   child: Center(
                    //     child: MockupPageList(
                    //       currentIndex: widget.controller.currentPage,
                    //       pages: widget.controller.pages,
                    //     ),
                    //   ),
                    // ),
                    // PageInfoCard(controller: controller),
                    Visibility(
                      visible: controller.pageMenuVisible,
                      maintainAnimation: true,
                      maintainState: true,
                      child: AnimatedOpacity(
                        // If the widget is visible, animate to 0.0 (invisible).
                        // If the widget is hidden, animate to 1.0 (fully visible).
                        opacity: controller.pageMenuVisible ? 1.0 : 0.0,
                        duration: const Duration(milliseconds: 500),
                        // The green box must be a child of the AnimatedOpacity widget.
                        child: Center(
                          child: MockupPageList(
                            controller: controller,
                            // onPageSelected: (value) {
                            //   setState(() {
                            //     // controller.menuVisible = !controller.menuVisible;
                            //     print('state changed');
                            //     menuVisible = !menuVisible;
                            //   });
                            // },
                          ),
                        ),
                      ),
                    ),
                  ],
                );
              },
            ),
            // child: Stack(
            //   children: [
            //     PageView.builder(
            //       pageSnapping: true,
            //       itemCount: controller.pages.length,
            //       controller: controller.pageController,
            //       itemBuilder: (context, index) {
            //         controller.setCurrentIndex(index);

            //         return controller.getBuilder(index)(context);
            //       },
            //     ),
            //     // AnimatedOpacity(
            //     //   // If the widget is visible, animate to 0.0 (invisible).
            //     //   // If the widget is hidden, animate to 1.0 (fully visible).
            //     //   opacity: menuVisible ? 1.0 : 0.0,
            //     //   duration: const Duration(milliseconds: 500),
            //     //   // The green box must be a child of the AnimatedOpacity widget.
            //     //   child: Center(
            //     //     child: MockupPageList(
            //     //       currentIndex: widget.controller.currentPage,
            //     //       pages: widget.controller.pages,
            //     //     ),
            //     //   ),
            //     // ),
            //     Visibility(
            //       visible: showPageInfos,
            //       maintainAnimation: true,
            //       maintainState: true,
            //       child: AnimatedOpacity(
            //         opacity: controller.pageInfosVisible ? 1.0 : 0.0,
            //         duration: const Duration(milliseconds: 800),
            //         child: Container(
            //           alignment: Alignment.topCenter,
            //           constraints: BoxConstraints(minWidth: MediaQuery.of(context).size.width),
            //           // height: 120,
            //           width: double.infinity,
            //           child: Material(
            //             child: Padding(
            //               padding: const EdgeInsets.all(8.0),
            //               child: ListTile(
            //                 title: Text(controller.currentPage.title),
            //                 leading: CircleAvatar(
            //                   child: Text(controller.currentPage.number.toString()),
            //                 ),
            //               ),
            //             ),
            //           ),
            //         ),
            //       ),
            //     ),
            //     Visibility(
            //       visible: controller.pageMenuVisible,
            //       maintainAnimation: true,
            //       maintainState: true,
            //       child: AnimatedOpacity(
            //         // If the widget is visible, animate to 0.0 (invisible).
            //         // If the widget is hidden, animate to 1.0 (fully visible).
            //         opacity: controller.pageMenuVisible ? 1.0 : 0.0,
            //         duration: const Duration(milliseconds: 500),
            //         // The green box must be a child of the AnimatedOpacity widget.
            //         child: Center(
            //           child: MockupPageList(
            //             controller: controller,
            //             // onPageSelected: (value) {
            //             //   setState(() {
            //             //     // controller.menuVisible = !controller.menuVisible;
            //             //     print('state changed');
            //             //     menuVisible = !menuVisible;
            //             //   });
            //             // },
            //           ),
            //         ),
            //       ),
            //     ),
            //   ],
            // ),
          );
        },
      ),
    );
  }
}

class PageInfoCard extends StatefulWidget {
  const PageInfoCard({
    Key? key,
    required this.controller,
  }) : super(key: key);

  final MockupController controller;

  @override
  State<PageInfoCard> createState() => _PageInfoCardState();
}

class _PageInfoCardState extends State<PageInfoCard> {
  late double hide;

  @override
  void initState() {
    super.initState();
    hide = 1.0;
  }

  @override
  Widget build1(BuildContext context) {
    return Visibility(
      visible: widget.controller.pageInfosVisible,
      maintainAnimation: true,
      maintainState: true,
      child: AnimatedOpacity(
        opacity: widget.controller.pageInfosVisible ? 1.0 : hide,
        duration: const Duration(milliseconds: 800),
        child: Container(
          alignment: Alignment.topCenter,
          constraints: BoxConstraints(minWidth: MediaQuery.of(context).size.width),
          // height: 120,
          width: double.infinity,
          child: Material(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Card(
                child: ListTile(
                  title: Text(widget.controller.currentPage.title),
                  contentPadding: EdgeInsets.all(16),
                  leading: CircleAvatar(
                    child: Text(widget.controller.currentPage.number.toString()),
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    Future.delayed(Duration(seconds: 1), () {
      if (mounted)
        setState(() {
          hide = 0.0;
        });
      ;
    });

    return Visibility(
      visible: widget.controller.pageInfosVisible,
      maintainAnimation: true,
      maintainState: true,
      child: AnimatedOpacity(
        opacity: hide,
        duration: const Duration(milliseconds: 800),
        child: Container(
          alignment: Alignment.topCenter,
          constraints: BoxConstraints(minWidth: MediaQuery.of(context).size.width),
          // height: 120,
          width: double.infinity,
          child: Material(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Card(
                child: ListTile(
                  title: Text(widget.controller.currentPage.title),
                  contentPadding: EdgeInsets.all(16),
                  leading: CircleAvatar(
                    child: Text(widget.controller.currentPage.number.toString()),
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
